using DomainClasses;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.Extensions.Logging;
using System;
using SharedKernel;
using System.Linq;

namespace Data
{
    public class PeopleContext : DbContext
    {

        //note: one of the ctors needs to be commented out in order
        //to use the EF Core Power Tools
        public PeopleContext(DbContextOptions options) : base(options)
        {
            SavingChanges += SetUserId;
        }

        public PeopleContext()
        {
            SavingChanges += SetUserId;
        }

        private void SetUserId(object sender, SavingChangesEventArgs e)
        {
            foreach (var entry in ChangeTracker.Entries().Where(entry => entry.Metadata.FindProperty("UserId") != null))
            {

                entry.Property("UserId").CurrentValue = Globals.CurrentUserId;
            }
        }


        public DbSet<Person> People { get; set; }
        public DbSet<Address> Addresses { get; set; }
        public DbSet<WildlifeSighting> WildlifeSightings { get; set; }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {


                var myconnectionstring = "Data Source = (localdb)\\MSSQLLocalDB; Initial Catalog = PeopleDb5;ConnectRetryCount=0";
                optionsBuilder.UseSqlServer(myconnectionstring)
                    .LogTo(Console.WriteLine, new[] { DbLoggerCategory.Database.Command.Name },
                           LogLevel.Information)
                    .EnableSensitiveDataLogging();

            }
            
           optionsBuilder.AddInterceptors(new QueryCommandInterceptor());

    }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Person>().HasQueryFilter(p => !p.IsDeleted);
            foreach (var e in modelBuilder.Model.GetEntityTypes().AsEnumerable().Where(e => e.IsPropertyBag == false & e.IsOwned() == false))
            {

                modelBuilder.Entity(e.ClrType).Property<int>("UserId");
            }
        }
    }

}
